/*
 **************************************************************************
 *
 *       DoSpecial for HP_Deskjet_400C driver.
 *       By PJ Hutchison 03/2/98
 *
 **************************************************************************
 */

#include "global.h"

#define LPI             9    /* Position in init.. string from 0 - 7 */
#define CPI             21
#define QUALITY         37   /* New LP_DJ command moved to end of string */
#define ASPECT          47   /* Portrait or Landscape */
#define INIT_LEN        49
#define LPP             10   /* Lines Per Page  */
#define FORM_LEN        14
#define LEFT_MARG       3
#define RIGHT_MARG      10
#define MARG_LEN        15

const char __ver[35] = "$VER: Deskjet_400C 40.17 (23.5.04)";

extern struct PrinterData *PD;
extern struct PrinterExtendedData *PED;


/* Routine to handle complex Escape sequences */

int DoSpecial(UWORD *command, char outputBuffer[], BYTE *vline, BYTE *currentVMI, BYTE *crlfFlag, UBYTE Parms[])
{

        static UWORD textlength, topmargin;
        int x, y, j, papersize, lpi;

        /* Initialise string:
           NB: Colour text mode is not supported on HP 400s!
           
           00-05  \033&l26A   A4 paper
           06-10  \033&l6D    6 lines per inch (LPI)
           11-14  \033&d@     Turn off underline
           15-19  \033(s0b    Bold off
           20-22       10h    10 CPI
           23-24        0p    Fixed spacing
           25-26        0s    Upright style
           27-29       12V    12/72 char height
           30-33  \033(1E     UK Char Set
           34-38  \033(s1Q    Quality (draft)
           39-43  \033&l0O    Portrait orientation
           44-48  \033(s3T    Courier Typeface
        */
        static char initThisPrinter[INIT_LEN + 1] =
         "\033&l26A\033&l6D\033&d@\033(s0b10h0p0s12V\033(1E\033(s1Q\033&l0O\033(s3T";
        
        /* Top margin, text length */        
        static char initForm[FORM_LEN + 1] = "\033&l000E\033&l000F";
        /* Left, right margins */
        static char initMarg[MARG_LEN + 1] = "\033&a000L\033&a000M\015";
        static char initTMarg[] = "\033&l000E\033&l000F";

        x = y = j = 0;

        if (*command == aRIN) {         /* Initialise command */
                while(x < INIT_LEN) {
                        outputBuffer[x] = initThisPrinter[x];
                        x++;
                }
                outputBuffer[x++] = '\015'; /* CR */

                lpi = 6;
                if (PD->pd_Preferences.PrintSpacing == EIGHT_LPI) {
                        outputBuffer[LPI] = '8';
                        lpi = 8;
                }

                if (PD->pd_Preferences.PrintPitch == ELITE) {
                        outputBuffer[CPI] = '2';
                }
                else if (PD->pd_Preferences.PrintPitch == FINE) {
                        outputBuffer[CPI] = '5';
                }

                if (PD->pd_Preferences.PrintQuality == LETTER) {
                        outputBuffer[QUALITY] = '2';
                }
                /* Change to Landscape (Vert)) if selected) */
                if (PD->pd_Preferences.PrintAspect == ASPECT_VERT) {
                            outputBuffer[ASPECT] = '1';
                }

                j = x; /* set the formlength = textlength, top margin = 0 */
                papersize = PD->pd_Preferences.PaperSize;
		switch (papersize) 
		{
		   case US_LETTER:
			papersize = 10 * lpi;
			break;
		   case US_LEGAL:
			papersize = 13 * lpi;
			break;
		   case EURO_A4:
			papersize = (lpi == 8 ? 85 : 65);
			break;
		   default:   /* Custom size */
			papersize = PD->pd_Preferences.PaperLength;
		}
                textlength = papersize;
                topmargin = 0;

                while (y < FORM_LEN) {
                        outputBuffer[x++] = initForm[y++];
                }
                numberString(textlength, j + LPP, outputBuffer);

                Parms[0] = PD->pd_Preferences.PrintLeftMargin;
                Parms[1] = PD->pd_Preferences.PrintRightMargin;
                *command = aSLRM;
        }

        if (*command == aSLRM) {        /* L&R Margins */
                j = x;
                y = 0;
                while(y < MARG_LEN) {
                        outputBuffer[x++] = initMarg[y++];
                }
                numberString(Parms[0] - 1, j + LEFT_MARG, outputBuffer);
                numberString(Parms[1] - 1, j + RIGHT_MARG, outputBuffer);
                return(x);
        }


        if(*command == aSUS0) {              /* Normalise the line */
                if (*vline > 0) {
                        *command = aPLD;
                }
                if (*vline < 0) {
                        *command = aPLU;
                }
                *vline = 0;
                return(0);
        }

        if (*command == aPLU) {
                (*vline)++;
                return(0);
        }

        if (*command == aPLD){
                (*vline)--;
                return(0);
        }

        if (*command == aSTBM) {        /* Top & bottom margins */
                if (Parms[0] == 0) {
                        Parms[0] = topmargin;
                }
                else {
                        topmargin = --Parms[0];
                }

                if (Parms[1] == 0) {
                        Parms[1] = textlength;
                }
                else {
                        textlength=Parms[1];
                }
                while (x < 13) {
                        outputBuffer[x] = initTMarg[x];
                        x++;
                }
                numberString(Parms[0], 3, outputBuffer);
                numberString(Parms[1] - Parms[0], 10, outputBuffer);
                return(x);
        }

        if (*command == aSLPP) {        /* Set form length */
                while(x < FORM_LEN) {
                        outputBuffer[x] = initForm[x];
                        x++;
                }
                /*restore textlength, margin*/
                numberString(topmargin, 3, outputBuffer);
                numberString(textlength, 10, outputBuffer);
                return(x);
        }

        if (*command == aRIS) {         /* Reset */
                PD->pd_PWaitEnabled = 253;   /* Save data! */
        }

        return(0);
}

/* Inserts Param into output buffer */
void numberString(UBYTE Param, int x, char outputBuffer[])
{
        if (Param > 199) {
                outputBuffer[x++] = '2';
                Param -= 200;
        }
        else if (Param > 99) {
                outputBuffer[x++] = '1';
                Param -= 100;
        }
        else {
                outputBuffer[x++] = '0'; /* always return 3 digits */
        }

        if (Param > 9) {
                outputBuffer[x++] = Param / 10 + '0';
        }
        else {
                outputBuffer[x++] = '0';
        }

        outputBuffer[x] = Param % 10 + '0';
}

/* Following functions are refered via printertag.asm */
   
int ConvFunc(char *buf, char c, int flag)
/* expand lf into lf/cr flag (0-yes, else no ) */
{
        if (c == '\014') { /* if formfeed (page eject) */
                PED->ped_PrintMode = 0; /* no data to print */
        }
        return(-1); /* pass all chars back to the printer device */
}

int Close(union printerIO *ior)
{
        if (PED->ped_PrintMode) { /* if data has been printed */
                (*(PD->pd_PWrite))("\014",1); /* eject page */
                (*(PD->pd_PBothReady))(); /* wait for it to finish */
                PED->ped_PrintMode = 0; /* no data to print */
        }
        return(0);
}
